#![feature(diagnostic_namespace)]
use crate::error_template::{AppError, ErrorTemplate};
use leptos::*;
use leptos_meta::*;
use leptos_router::*;

use crate::components;
use crate::pages;
use crate::util;
use leptos_toaster::{Toaster, ToasterPosition};

#[component]
pub fn App() -> impl IntoView {
    // Provides context that manages stylesheets, titles, meta tags, etc.
    provide_meta_context();
    let participants: util::surrealdb::schemas::ParticipantsContext = create_rw_signal(vec![]);
    let participant: RwSignal<Option<String>> = create_rw_signal(None);

    provide_context(participant);

    provide_context(participants);
    util::surrealdb::init_participants();
    util::websocket::client::init_websocket();

    view! {
        // injects a stylesheet into the document <head>
        // id=leptos means cargo-leptos will hot-reload this stylesheet
        <Stylesheet id="leptos" href="/pkg/application.css"/>

        // sets the document title
        <Title text="WRB Timings"/>

        // content for this welcome page
        <Router fallback=|| {
            let mut outside_errors = Errors::default();
            outside_errors.insert_with_default_key(AppError::NotFound);
            view! {
                <ErrorTemplate outside_errors/>
            }
            .into_view()
        }>
            <Toaster
                position=ToasterPosition::BottomCenter
            >
                <components::header::Header />
                <components::participant::Modal />
                <main>
                    <Routes>
                        <Route path="/" view=pages::index::HomePage />
                        <Route path="/add-participant" view=pages::add_participant::AddParticipant />
                        <Route path="/add-time" view=pages::add_time::AddTime />
                        <Route path="/groups" view=pages::groups::Groups />
                    </Routes>
                </main>
            </Toaster>
        </Router>
    }
}
